{% extends "getpaid/base.html" %}
{% block content %}
<h1>django-getpaid example project</h1>
<p>This is an example project of how django-getpaid manage order payment workflow.</p>
<p>First you will need to create an order. In this simple example order has just one item which have
name and price.</p>

<h2>Please fill the form to create an order</h2>
<form action="" method="post">
    {% csrf_token %}
    <table>
    {{ form.as_table }}
    </table>
    <input type="submit" value="Place your order">
</form>
{% if orders %}
    <h2>Orders</h2>
    <ol>
        {% for order in orders %}
            <li> <a href="{{ order.get_absolute_url }}">{{ order }}</a>
        {% endfor %}
    </ol>
{% endif %}
{% endblock content %}
