# Generated by Django 3.0.5 on 2020-04-12 19:28

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='PaymentEntry',
            fields=[
                (
                    'id',
                    models.UUIDField(
                        default=uuid.uuid4, primary_key=True, serialize=False
                    ),
                ),
                ('payment', models.CharField(max_length=100)),
                ('value', models.DecimalField(decimal_places=2, max_digits=20)),
                ('currency', models.CharField(max_length=3)),
                ('description', models.TextField()),
                ('callback', models.URLField()),
                ('success_url', models.URLField()),
                ('failure_url', models.URLField()),
            ],
        ),
    ]
