# Generated by Django 3.0.5 on 2020-04-19 15:00

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('paywall', '0002_auto_20200417_2107'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymententry',
            name='callback',
            field=models.URLField(blank=True),
        ),
        migrations.AlterField(
            model_name='paymententry',
            name='description',
            field=models.TextField(blank=True),
        ),
        migrations.AlterField(
            model_name='paymententry',
            name='failure_url',
            field=models.URLField(blank=True),
        ),
        migrations.AlterField(
            model_name='paymententry',
            name='success_url',
            field=models.URLField(blank=True),
        ),
    ]
