{% extends "getpaid/base.html" %}
{% load i18n %}

{% block content %}
<h1>django-getpaid fake payment gateway</h1>
<h3>This page simulates the broker's paywall where usually you would
  proceed with payment by entering card data or something.</h3>
<p>{{ message }}</p>
<p>
    {% blocktrans %}
    Are you authorizing payment "<strong>{{ ext_id }}</strong>"
    valued to <strong>{{ value }} {{ currency }}</strong>
    for "<strong>{{ description }}</strong>"?
    {% endblocktrans %}
</p>
<form method="post">
    {{ form.as_p }}
    <input type="submit" value="{% trans "Continue" %}">
</form>
{% endblock content %}
